/**
 * $Id: callbacks.c,v 1.8 2001/05/16 21:34:26 makki Exp $
 * $Author: makki $
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef HAVE_GNOME
#include <gnome.h>
#endif

#include <gnome-xml/tree.h>
#include <glade/glade.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "gtkxmlcontainer.h"
#include "filedlg.h"
#include "gtkxmlmodules.h"

void do_new( GtkXmlContainer* container, xmlDoc* doc );
void on_fileopen_ok_clicked( GtkWidget* widget, gpointer user_data );

extern char* filename;
extern GtkXmlModules* modules;

GtkXmlContainer* e;

editable_insert_nospace(GtkEditable *editable,
								const gchar* text,
								gint length,
								gint *position,
								gpointer data)
{
	int i,j = 0;
	gchar* result = g_new(gchar, length);
	gint newlength = length;
	for( i=0; i<length; i++){
		switch( text[i] ){
			case ' ': case '\t': case '\n': case '\r':
				newlength--;
			break;
			default:
				result[j] = text[i];
				j++;
			break;
		}
	}
	if( newlength ){
		gtk_signal_handler_block_by_func(	GTK_OBJECT(editable),
														GTK_SIGNAL_FUNC(editable_insert_nospace),
														data );
		gtk_editable_insert_text( editable, result, newlength, position );
		gtk_signal_handler_unblock_by_func( GTK_OBJECT(editable),
														GTK_SIGNAL_FUNC(editable_insert_nospace),
														data );
	}
	gtk_signal_emit_stop_by_name( GTK_OBJECT(editable), "insert_text" );
	g_free(result);
}

void
do_new( GtkXmlContainer* container, xmlDoc* doc ){
	filename = NULL;
	doc = gtk_xml_container_set_document(container,doc);
	if( doc != NULL ){
		xmlFreeDoc(doc);
	}
}
void
do_open( GtkXmlContainer* container, gchar* fname ){
	xmlDocPtr doc;

	doc = (xmlDocPtr)xmlParseFile( (char*)fname );
	if( doc == NULL ){
		show_error("Reading of file failed. Is this a valid XML document ?");
		return;
	}
	doc = gtk_xml_container_set_document(container,doc);
	if( doc != NULL ){
		xmlFreeDoc(doc);
	}
	if( filename != NULL ){
		g_free(filename);
	}
	filename = fname;
}
static void
do_save( GtkXmlContainer* container, char* fname ){
	GtkWidget* widget;
	if( fname == NULL ){
		widget = filedlg_new();
		fname = filedlg_show(widget, fname);
		if( filename != NULL ){
			g_free(filename);
			filename = NULL;
		}
		gtk_widget_destroy(widget);
	}
	if( fname != NULL ){
		xmlSaveFile(fname,gtk_xml_container_get_document(container) );
		if( filename == NULL ){
			filename = fname;
		}
	}
}

static void
do_saveas( GtkXmlContainer* container ){
	do_save( container, NULL );
}



/** SIGNALS */

void
on_new_file1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget* w = newdlg_new( e );
	xmlDoc* doc = newdlg_show(w);
	if( doc != NULL ){
		do_new( e, doc );
	}
}

void
on_open1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget* w = filedlg_new();
	char* filename = filedlg_show(w,NULL);
	if( filename != NULL ){
		do_open(e,filename);
	}
}
void
on_openbutton1_clicked	(GtkButton* button, gpointer user_data ){
	GtkWidget* w = filedlg_new();
	char* filename = filedlg_show(w,NULL);
	if( filename != NULL ){
		do_open(e,filename);
	}
}
/*
void
on_open1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget* widget;
	GladeXML* xml = glade_xml_new(GLADE_UI_FILE,"fileopen");
	glade_xml_signal_autoconnect(xml);
	widget = glade_xml_get_widget(xml, "fileopen");
	gtk_widget_show(widget);
	widget = glade_xml_get_widget(xml, "fileopen_ok");
	gtk_signal_connect( GTK_OBJECT(widget), "clicked", on_fileopen_ok_clicked, user_data );
}
void
on_openbutton1_clicked	(GtkButton* button, gpointer user_data ){
	GtkWidget* widget;
	GladeXML* xml = glade_xml_new(GLADE_UI_FILE,"fileopen");
	glade_xml_signal_autoconnect(xml);
	widget = glade_xml_get_widget(xml, "fileopen");
	gtk_widget_show(widget);
	widget = glade_xml_get_widget(xml, "fileopen_ok");
	gtk_signal_connect( GTK_OBJECT(widget), "clicked", on_fileopen_ok_clicked, user_data );
}
*/
void
on_savebutton1_clicked( GtkButton* button, gpointer user_data ){
	do_save(e,filename);
}

void
on_save1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	do_save(e,filename);
}

void
on_save_as1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	do_saveas(e);
}


void
on_exit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gtk_main_quit();
}


void
on_cut1_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_copy1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_paste1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_clear1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}

void
on_add_element_node1_activate				(GtkMenuItem *menuitem,
													gpointer user_data)
{
	GtkXmlView* view;
	xmlNode* node;
	
	view = (GtkXmlView *)gtk_xml_container_get_view(e);
	node = gtk_xml_container_get_current_node( e );
	if( node != NULL ){
		gtk_xml_view_append_child(view, xmlNewNode(NULL, "Untitled"),node);
	}else{
		xmlDoc* doc = gtk_xml_container_get_document(e);
		if( doc == NULL ){
			return;
		}
		if( xmlDocGetRootElement(doc) == NULL ){
			gtk_xml_view_append_child(view, xmlNewNode(NULL, "Untitled"),(xmlNode*)doc);
		}else{
			gdk_beep();
		}
	}
}

void
on_add_text_node1_activate					(GtkMenuItem *menuitem,
													gpointer user_data)
{
	xmlNode* node = gtk_xml_container_get_current_node( e );
	GtkXmlView* view = (GtkXmlView *)gtk_xml_container_get_view(e);
	if( node != NULL ){
		gtk_xml_view_append_child(view, xmlNewText(""),node);
	}else{
		return;
	}
}
void
on_delete_node1_activate					(GtkMenuItem* menuitem,
													gpointer user_data)
{
	xmlNode* node = gtk_xml_container_get_current_node( e );
	GtkXmlView* view = (GtkXmlView *)gtk_xml_container_get_view(e);
	if( node != NULL ){
		gtk_xml_view_delete_node(view, node );
	}
}


void
on_properties1_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GladeXML* xml = glade_xml_new(GLADE_UI_FILE,"editprops");
	GtkWidget* window;
	GtkWidget* widget;
	char* text;
	xmlDoc* doc = gtk_xml_container_get_document(e);
	xmlDtd* dtd = doc->intSubset;
	xmlNs* ns = doc->oldNs;
	glade_xml_signal_autoconnect(xml);
	window = glade_xml_get_widget(xml, "editprops");
	gtk_object_set_data( GTK_OBJECT(window),"gladexml",xml);
	if( dtd ){
		widget = glade_xml_get_widget(xml, "name_entry");
		if( dtd->name ){
			gtk_entry_set_text( GTK_ENTRY(widget), dtd->name );
		}
		widget = glade_xml_get_widget(xml, "externalid_entry");
		if( dtd->ExternalID ){
			gtk_entry_set_text( GTK_ENTRY(widget), dtd->ExternalID);
		}
		widget = glade_xml_get_widget(xml, "systemid_entry");
		if( dtd->SystemID ){
			gtk_entry_set_text( GTK_ENTRY(widget), dtd->SystemID);
		}
	}
	if( ns ){
		widget = glade_xml_get_widget(xml, "prefix_entry");
		if( ns->prefix ){
			gtk_entry_set_text( GTK_ENTRY(widget), ns->prefix );
		}
		widget = glade_xml_get_widget(xml, "href_entry");
		if( ns->href ){
			gtk_entry_set_text( GTK_ENTRY(widget), ns->href );
		}
	}
	gtk_widget_show(window);
}


void
on_preferences1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}

void
on_application_delete_event( GtkWidget* widget, gpointer user_data ){
	gtk_main_quit();

}
void
on_application_destroy_event( GtkWidget* widget, gpointer user_data ){
	gtk_main_quit();

}


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GladeXML* xml = glade_xml_new(GLADE_UI_FILE,"about");
	GtkWidget* about;
	glade_xml_signal_autoconnect(xml);
	about = glade_xml_get_widget(xml, "about");
	gtk_widget_show(about);
}

void
on_newbutton1_clicked( GtkWidget* widget, gpointer user_data ){
	GtkWidget* w = newdlg_new(e);
	xmlDoc* doc = newdlg_show(w);
	if( doc != NULL ){
		do_new( e, doc );
	}
}

/** Dialogs */
/*
void
on_fileopen_ok_clicked( GtkWidget* widget, gpointer user_data ){
	GtkWidget* w = gtk_widget_get_toplevel( widget );
	if( w != NULL ){
		GtkFileSelection* sel = GTK_FILE_SELECTION(w);
		char* str = gtk_file_selection_get_filename( sel );
		if( str != NULL ){
			str = strdup(str);
		}
		do_open( e, str );
		gtk_widget_destroy(GTK_WIDGET(sel));
	}
}
*/
void
on_dialog_close_clicked( GtkWidget* widget, gpointer user_data ){
	GtkWidget* w = gtk_widget_get_toplevel( widget );
	gtk_widget_destroy(w);
}
void
on_editprops_ok_clicked( GtkWidget* w, gpointer user_data ){
	GtkWidget* widget;
	GladeXML* xml = (GladeXML *)gtk_object_get_data(
             GTK_OBJECT(gtk_widget_get_toplevel(w)), "gladexml");
	if( xml ){
		char* dtdname;
		char* dtdeid;
		char* dtdsysid;
		xmlDtd* dtd;
		xmlDoc* doc = gtk_xml_container_get_document(e);
		widget = glade_xml_get_widget(xml, "name_entry");
		if( widget ){
			dtdname = gtk_entry_get_text( GTK_ENTRY(widget) );
		}else{
			show_error("dtdname widget not found\n");
		}
		widget = glade_xml_get_widget(xml, "externalid_entry");
		dtdeid = gtk_entry_get_text( GTK_ENTRY(widget));
		if( widget ){
			widget = glade_xml_get_widget(xml, "dtdsysid_entry");
		}else{
			show_error("externalid widget not found\n");
		}
		widget = glade_xml_get_widget(xml, "systemid_entry");
		if( widget ){
			dtdsysid = gtk_entry_get_text( GTK_ENTRY(widget));
		}else{
			show_error("systemid widget not found\n");
		}
		if( doc->intSubset ){
			xmlFreeDtd( doc->intSubset );
			doc->intSubset = xmlNewDtd( doc, dtdname, dtdeid, dtdsysid );
			if( doc->intSubset == NULL ){
				show_error("unable to set intSubset\n");
			}
		}else{
			xmlCreateIntSubset( doc, dtdname, dtdeid, dtdsysid );
		}
	}
	gtk_widget_destroy( gtk_widget_get_toplevel(w));
}



/** Widgets */

GtkWidget*
mycomponent_new (gchar *widget_name, gchar *string1, gchar *string2,
                gint int1, gint int2)
{
	/*
	xmlDocPtr doc;
	gchar* mod;
	if( filename != NULL ){
		doc = (xmlDocPtr)xmlParseFile(filename);
	}else{
		doc = xmlNewDoc("1.0");
	}
	mod = gtk_xml_modules_find_container( modules, doc );
	if( mod != NULL ){
		ModuleInfo* info = gtk_xml_modules_get_module_info( modules, mod );
		GtkWidget* (*func) (xmlDocPtr);
		if( g_module_symbol( info->module, "module_create_container", &func ) ){
			GtkWidget* w;
			w = GTK_WIDGET( GTK_XML_EDITOR(func( doc )) );
			return w;
		}
	}
	return NULL;
	*/

	return e = gtk_xml_container_new(modules);
	/*
	xmlDocPtr doc;
	gchar* mod;

	e = c;
	mod = gtk_xml_modules_find_editor( modules, doc );
	if( mod != NULL ){
		ModuleInfo* info = gtk_xml_modules_get_module_info( modules, mod );
		GtkWidget* (*func) (GtkXmlContainer* c);
		if( g_module_symbol( info->module, "module_create_editor", &func ) ){
			GtkWidget* w;
			w = GTK_WIDGET( func(c) );
			gtk_xml_container_set_editor( c, w );
			mod = gtk_xml_modules_find_view( modules, doc );
			if( mod != NULL ){
				GtkXmlView* (*func2) (GtkXmlContainer*, GtkWidget* );
				info = gtk_xml_modules_get_module_info( modules, mod );
				if( g_module_symbol( info->module, "module_create_view", &func ) ){
					w = GTK_WIDGET( func2(c,w) );
					gtk_xml_container_set_view( c, w );
				}
			}
		}
	}
	if( filename != NULL ){
		doc = (xmlDocPtr)xmlParseFile(filename);
	}else{
		doc = xmlNewDoc("1.0");
	}
	gtk_xml_container_set_document( c, doc );
	return c;
	*/
	/*
        xmlDocPtr doc;
	if( filename != NULL ){
		doc = (xmlDocPtr)xmlParseFile( filename );
	}else{
		doc = (xmlDocPtr)NULL;
	}
	e = GTK_XML_EDITOR(gtk_xml_container_new((xmlDoc*)doc));
	gtk_widget_show( GTK_WIDGET(e) );
	return GTK_WIDGET(e);
	*/
}


/** ***************** Modules ********************
 */
void
on_module_properties1_activate( GtkMenuItem *menuitem,
                                        gpointer         user_data)
{
	GtkWidget* widget;
	GtkWidget* widget2;
	GList* list;
	widget = gtk_xml_modules_get_configuration( modules );
	if( widget == NULL ){
		show_error("warning: configuration widget not found for modules\n");
		return;
	}
	gtk_widget_show(widget);
}
void
on_module_enable_toggled( GtkCheckButton* widget ){
	g_print("module_enable");
}
void
on_module_add_clicked( GtkButton* button, gpointer user_data ){
	GtkWidget* widget;
	GladeXML* xml = glade_xml_new(GLADE_UI_FILE,"fileopen");
	glade_xml_signal_autoconnect(xml);
	widget = glade_xml_get_widget(xml, "fileopen");
	gtk_widget_show(widget);
}
