/**
 * $Id: gtkxmleditor.c,v 1.9 2001/05/16 21:37:57 makki Exp $
 * $Author: makki $
 */



#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <glade/glade.h>
#include <gnome-xml/tree.h>
#include "gtkxmleditor.h"
#include "gtkxmlcontainer.h"

extern GladeXML *gladeUI;

struct _GtkXmlEditorPrivate{
	GtkXmlContainer* container;
	GtkWidget* pane;
	GtkWidget* left;
	GtkWidget* notebook;
	GtkWidget* name_entry;
	GtkWidget* attr_name_entry;
	GtkWidget* attr_value_entry;
	GtkWidget* text;
	GtkWidget* attributes;
};
typedef struct _GtkXmlEditorPrivate GtkXmlEditorPrivate;

enum{
	CHANGED,
	SIGNAL_NUM
};

static void gtk_xml_editor_class_init( GtkXmlEditorClass *klass);
static void gtk_xml_editor_init( GtkXmlEditor *editor);
void _xml_select_node_sig( GtkXmlView* view, xmlNode* node, gpointer user_data );
void _xml_update_node_name_widget_sig( GtkEntry* entry, gpointer user_data );
void _xml_update_node_text_widget_sig( GtkText* text, gpointer user_data );
void _xml_update_node_sig( GtkXmlView* view, xmlNode* node, gpointer user_data );
void _xml_update_attr_widget_sig( GtkButton* button, gpointer user_data );
void _xml_add_attr_widget_sig( GtkButton* button, gpointer user_data);
void _xml_delete_attr_widget_sig( GtkButton* button, gpointer user_data);
void _xml_list_select_widget_sig( GtkCList* list, gint row, gint column, GdkEventButton *event, gpointer user_data );
static void _xml_change_view( GtkXmlContainer* editor, GtkXmlView* view, gpointer data );
static void gtk_xml_editor_destroy( GtkObject *object);

static GtkBoxClass *parent_class = NULL;
static guint xml_editor_signals [SIGNAL_NUM] = {0};

guint
gtk_xml_editor_get_type(void){
	static guint gtk_xml_editor_type = 0;
	if(!gtk_xml_editor_type){
		static const GtkTypeInfo gtk_xml_editor_info = {
			"GtkXmlEditor",
			sizeof (GtkXmlEditor),
			sizeof (GtkXmlEditorClass),
			(GtkClassInitFunc) gtk_xml_editor_class_init,
			(GtkObjectInitFunc) gtk_xml_editor_init,
			NULL,
			NULL,
			(GtkClassInitFunc) NULL
		};
		gtk_xml_editor_type = gtk_type_unique(gtk_hbox_get_type(), &gtk_xml_editor_info);
	}
	return gtk_xml_editor_type;
}

static void
gtk_xml_editor_class_init(GtkXmlEditorClass* klass){
	GtkObjectClass* object_class;
	object_class = (GtkObjectClass*)klass;
	parent_class = gtk_type_class(GTK_TYPE_HBOX);
	object_class->destroy = gtk_xml_editor_destroy;
	xml_editor_signals[CHANGED] = gtk_signal_new("changed",
												GTK_RUN_FIRST,
												object_class->type,
												GTK_SIGNAL_OFFSET(GtkXmlEditorClass, changed),
												gtk_marshal_NONE__NONE,
												GTK_TYPE_NONE, 0);
	gtk_object_class_add_signals(object_class, xml_editor_signals, SIGNAL_NUM);
	klass->changed = NULL;
}

static void
gtk_xml_editor_init( GtkXmlEditor* editor ){
	GtkXmlEditorPrivate* priv;
	GtkWidget* widget;
	GladeXML* xml;
	GTK_WIDGET_SET_FLAGS(GTK_WIDGET(editor), GTK_NO_WINDOW);

	priv = g_new0( GtkXmlEditorPrivate, 1);
	editor->private_data = priv;
	priv->pane = gtk_hpaned_new();
	gtk_box_pack_start_defaults( GTK_BOX(editor), priv->pane );

	xml = glade_xml_new(GLADE_GTKXMLEDITOR_FILE,"editor_notebook");
	glade_xml_signal_autoconnect(xml);
	priv->text = glade_xml_get_widget(xml, "nodetext");
	if( priv->text != NULL ){
		gtk_signal_connect( GTK_OBJECT(priv->text), "changed", _xml_update_node_text_widget_sig, editor);
	}
	priv->notebook = glade_xml_get_widget( xml, "editor_notebook" );
	if( priv->notebook != NULL ){
		gtk_paned_pack2( GTK_PANED(priv->pane), priv->notebook, TRUE, FALSE );
		gtk_widget_hide( priv->notebook );
	}else{
		show_error("warning: unable to find notebook\n");
	}
	widget = gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(widget), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
	gtk_paned_pack1( GTK_PANED(priv->pane), widget, TRUE, FALSE );
	gtk_widget_show(widget);
	priv->left = gtk_viewport_new(NULL,NULL);
	gtk_container_add( GTK_CONTAINER(widget), priv->left );
	gtk_widget_show( priv->left );
	
	priv->name_entry = glade_xml_get_widget( xml, "nodename");
	if( priv->name_entry == NULL ){
		show_error("WARNING: unable to find widget named 'nodename'\n");
	}else{
		if( !GTK_IS_ENTRY(priv->name_entry) ){
			show_error("WARNING: widget named 'nodename' is not a textentry\n");
			priv->name_entry = NULL;
		}else{
			gtk_signal_connect( GTK_OBJECT(priv->name_entry), "changed", _xml_update_node_name_widget_sig, editor );
		}
	}
	/** Properties */
	priv->attr_name_entry = glade_xml_get_widget(xml, "attrname");
	if( priv->attr_name_entry == NULL ){
		show_error("attrname widget not found\n");
	}
	priv->attr_value_entry = glade_xml_get_widget(xml, "attrvalue");
	if( priv->attr_value_entry == NULL ){
		show_error("attrvalue widget not foudn\n");
	}
	widget = glade_xml_get_widget(xml, "propadd");
	gtk_signal_connect( GTK_OBJECT(widget), "clicked", _xml_add_attr_widget_sig, editor );
	widget = glade_xml_get_widget(xml, "propupdate");
	gtk_signal_connect( GTK_OBJECT(widget), "clicked", _xml_update_attr_widget_sig, editor );
	widget = glade_xml_get_widget(xml, "propdelete");
	gtk_signal_connect( GTK_OBJECT(widget), "clicked", _xml_delete_attr_widget_sig, editor );
	priv->attributes = glade_xml_get_widget(xml, "attributes");
	gtk_signal_connect( GTK_OBJECT(priv->attributes), "select_row", GTK_SIGNAL_FUNC(_xml_list_select_widget_sig), editor );
}

GtkWidget*
gtk_xml_editor_new(GtkXmlContainer* container ){
	GtkXmlEditor* editor;
	GtkXmlEditorPrivate *priv;
	if( container == NULL ){
		return NULL;
	}
	editor = gtk_type_new(gtk_xml_editor_get_type());
	priv = (GtkXmlEditorPrivate*)editor->private_data;

	if( container != NULL ){
		priv->container = container;
		gtk_signal_connect( GTK_OBJECT(container), "view_changed", GTK_SIGNAL_FUNC(_xml_change_view ), editor );
	}
	
	gtk_widget_set_usize(GTK_WIDGET(editor),400,250);
	gtk_widget_show(priv->pane);
	return GTK_WIDGET(editor);
}

static void
_xml_change_view( GtkXmlContainer* container, GtkXmlView* view, gpointer data ){
	GtkXmlEditor* editor = GTK_XML_EDITOR(data);
	GtkXmlEditorPrivate* priv;
	g_assert(container);
	g_assert(view);
	g_assert(data);
	if( !IS_GTK_XML_VIEW(view) ){
		show_error("ERROR: view is not of type GTK_XML_VIEW\n");
		return;
	}
	if( !IS_GTK_XML_EDITOR(editor) ){
		show_error("ERROR: parameter not an editor\n");
		return;
	}
	gtk_xml_editor_set_view( editor, view );
}

static void
xml_editor_draw_node( GtkXmlEditor* editor, xmlNode* node ){
	GtkXmlEditorPrivate* priv = (GtkXmlEditorPrivate*)editor->private_data;
	xmlAttr* attrs;
	GtkWidget* widget;
	if( node != NULL ){
		char* line[2];
		gtk_widget_show( priv->notebook );
		switch(node->type){
			case XML_ELEMENT_NODE:
				gtk_text_set_editable( GTK_TEXT(priv->text), FALSE );
				widget = gtk_notebook_get_nth_page( GTK_NOTEBOOK(priv->notebook), 0 );
				gtk_widget_show(widget);
				if( priv->name_entry != NULL ){
					GtkEntry* entry = GTK_ENTRY( priv->name_entry );
					gtk_object_set_user_data( GTK_OBJECT( entry ), node );
					gtk_entry_set_text( entry, node->name );
				}
				gtk_clist_clear((GtkCList *)priv->attributes );
				attrs = node->properties;
				while( attrs != NULL ){
					line[0] = (char*)attrs->name;
					line[1] = (char*)attrs->val->content;
					gtk_clist_append( GTK_CLIST(priv->attributes), line );
					attrs = attrs->next;
				}
				widget = gtk_notebook_get_nth_page( GTK_NOTEBOOK(priv->notebook), 1 );
				gtk_widget_hide(widget);
			break;
			case XML_TEXT_NODE:
			case XML_CDATA_SECTION_NODE:
				widget = gtk_notebook_get_nth_page((GtkNotebook *)priv->notebook, 1 );
				gtk_widget_show(widget);
				gtk_object_set_user_data( GTK_OBJECT(priv->text), node );
				gtk_editable_delete_text( GTK_EDITABLE(priv->text), 0, -1);
				gtk_text_set_editable( GTK_TEXT(priv->text), TRUE );
				gtk_text_insert( GTK_TEXT(priv->text), NULL, NULL, NULL, node->content, -1 );
				widget = gtk_notebook_get_nth_page( GTK_NOTEBOOK(priv->notebook), 0 );
				gtk_widget_hide(widget);
			break;
			default:
				g_print("UNKNOWN NODE TYPE %d\n", node->type);
			break;
		}
	}else{
		gtk_widget_hide(priv->notebook);
		gtk_editable_delete_text( GTK_EDITABLE(priv->text), 0, -1 );
		gtk_text_set_editable( GTK_TEXT(priv->text), FALSE );
		gtk_object_remove_data(GTK_OBJECT(priv->name_entry), 
                                       "user_data");
		gtk_entry_set_text( GTK_ENTRY( priv->name_entry), "" );
		gtk_clist_clear( GTK_CLIST( priv->attributes) );
	}
}

void
_xml_editor_enable_notebook( GtkXmlEditor* ed ){
	GtkXmlEditorPrivate* priv = (GtkXmlEditorPrivate*)ed->private_data;
	gtk_widget_show( priv->notebook );
}
void
_xml_editor_disable_notebook( GtkXmlEditor* ed ){
	GtkXmlEditorPrivate* priv = (GtkXmlEditorPrivate*)ed->private_data;
	gtk_widget_hide( priv->notebook );
}
void
_xml_update_node_name_widget_sig( GtkEntry* entry, gpointer data ){
	xmlNode* node = gtk_object_get_user_data( GTK_OBJECT(entry) );
	GtkXmlEditor* editor = GTK_XML_EDITOR(data);
	if( node != NULL ){
		gchar* name = gtk_entry_get_text(entry);
		if( strlen(name) > 0 ){
			xmlNodeSetName( node, gtk_entry_get_text( entry ) );
			gtk_signal_handler_block_by_func( GTK_OBJECT(entry), _xml_update_node_name_widget_sig, data );
			gtk_xml_view_update_node( GTK_XML_VIEW(editor->tree), node );
			gtk_signal_handler_unblock_by_func( GTK_OBJECT(entry), _xml_update_node_name_widget_sig, data );
		}
	}
}
void
_xml_update_node_text_widget_sig( GtkText* text, gpointer data ){
	xmlNode* node = gtk_object_get_user_data( GTK_OBJECT(text));
	GtkXmlEditor* editor = GTK_XML_EDITOR(data);
	if( node != NULL){
		gchar* block = gtk_editable_get_chars( GTK_EDITABLE(text), 0, -1 );
		if( strlen(block) > 0 ){
			xmlNodeSetContent( node, block );
			gtk_signal_handler_block_by_func( GTK_OBJECT(text), _xml_update_node_text_widget_sig, data );
			gtk_xml_view_update_node( GTK_XML_VIEW(editor->tree), node );
			gtk_signal_handler_unblock_by_func( GTK_OBJECT(text), _xml_update_node_text_widget_sig, data );
		}
	}
}

void
_xml_update_node_sig( GtkXmlView* view, xmlNode* node, gpointer user_data ){
	if( !node ){
		return;
	}
	if( node == gtk_xml_view_get_selected_node(view) ){
		xml_editor_draw_node( GTK_XML_EDITOR(user_data), node);
	}
}
void
_xml_editor_delete_node_sig( GtkXmlView* view, xmlNode* node, gpointer user_data ){
	GtkXmlEditor* editor = GTK_XML_EDITOR(user_data);
	GtkXmlEditorPrivate* priv = (GtkXmlEditorPrivate*)editor->private_data;
	xmlDoc* doc = gtk_xml_editor_get_document(editor);
	if( node == xmlDocGetRootElement(doc) ){
		xml_editor_draw_node(editor, NULL);
	}
}

void
_xml_select_node_sig( GtkXmlView* view, xmlNode* node, gpointer user_data ){
	GtkXmlEditor* editor = GTK_XML_EDITOR(user_data);
	GtkXmlEditorPrivate* priv = (GtkXmlEditorPrivate*)editor->private_data;
	if(!node){
		return;
	}
	xml_editor_draw_node(editor,node);
}

void _xml_update_attr_widget_sig( GtkButton* button, gpointer user_data ){
	GtkXmlEditor* editor = GTK_XML_EDITOR(user_data);
	GtkXmlEditorPrivate* priv = (GtkXmlEditorPrivate*)editor->private_data;
	xmlNode* current;
	if( ( current = gtk_xml_view_get_selected_node( GTK_XML_VIEW(editor->tree) )) != NULL ){
		gchar* cname = gtk_entry_get_text( GTK_ENTRY(priv->attr_name_entry));
		if( strlen( cname ) > 0 ){
			gchar* cvalue = gtk_entry_get_text( GTK_ENTRY(priv->attr_value_entry));
			if( strlen(cvalue) > 0 ){
				gtk_xml_view_update_attribute((GtkXmlView *)editor->tree, current, cname, cvalue );
				gtk_editable_delete_text( GTK_EDITABLE(priv->attr_name_entry), 0, -1 );
				gtk_editable_delete_text( GTK_EDITABLE(priv->attr_value_entry), 0, -1 );
				return;
			}
		}
	}
	gdk_beep();
}
void _xml_add_attr_widget_sig( GtkButton* button, gpointer user_data){
	GtkXmlEditor* editor = GTK_XML_EDITOR(user_data);
	GtkXmlEditorPrivate* priv = (GtkXmlEditorPrivate*)editor->private_data;
	xmlNode* current;
	if( (current=gtk_xml_view_get_selected_node( GTK_XML_VIEW(editor->tree))) != NULL ){
		gchar* cname = gtk_entry_get_text( GTK_ENTRY(priv->attr_name_entry));
		if( strlen( cname ) > 0 ){
			gchar* cvalue = gtk_entry_get_text( GTK_ENTRY(priv->attr_value_entry));
			if( strlen(cvalue) > 0 ){
				gtk_xml_view_add_attribute((GtkXmlView *)editor->tree, current, cname, cvalue );
				gtk_editable_delete_text( GTK_EDITABLE(priv->attr_name_entry), 0, -1 );
				gtk_editable_delete_text( GTK_EDITABLE(priv->attr_value_entry), 0, -1 );
				return;
			}
		}
	}
	gdk_beep();
}
void _xml_delete_attr_widget_sig( GtkButton* button, gpointer user_data){
	GtkXmlEditor* editor = GTK_XML_EDITOR(user_data);
	GtkXmlEditorPrivate* priv = (GtkXmlEditorPrivate*)editor->private_data;
	xmlNode* current;
	if( (current=gtk_xml_view_get_selected_node(GTK_XML_VIEW(editor->tree))) != NULL ){
		gchar* cname = gtk_entry_get_text( GTK_ENTRY(priv->attr_name_entry) );
		if( strlen( cname ) > 0 ){
			gtk_xml_view_delete_attribute( editor->tree, current, cname );
			gtk_editable_delete_text( GTK_EDITABLE(priv->attr_name_entry), 0, -1 );
			gtk_editable_delete_text( GTK_EDITABLE(priv->attr_value_entry), 0, -1 );
			return;
		}
	}
	gdk_beep();
}
void _xml_list_select_widget_sig( GtkCList* list, gint row, gint column, GdkEventButton *event, gpointer user_data ){
	GtkXmlEditor* editor = GTK_XML_EDITOR(user_data);
	GtkXmlEditorPrivate* priv = (GtkXmlEditorPrivate*)editor->private_data;
	char* text;
	gtk_clist_get_text( list, row, 0, &text );
	gtk_entry_set_text( GTK_ENTRY(priv->attr_name_entry), text );
	gtk_clist_get_text( list, row, 1, &text );
	gtk_entry_set_text( GTK_ENTRY(priv->attr_value_entry), text );
}
void
gtk_xml_editor_destroy( GtkObject* object ){
	GtkXmlEditor *editor;
	GtkXmlEditorPrivate *priv;

	g_return_if_fail(object);
	g_return_if_fail(IS_GTK_XML_EDITOR(object));
	editor = GTK_XML_EDITOR(object);
	g_free(editor->private_data);
	if( GTK_OBJECT_CLASS(parent_class)->destroy){
		(*GTK_OBJECT_CLASS(parent_class)->destroy)(object);
	}
}
xmlDoc*
gtk_xml_editor_get_document( GtkXmlEditor *editor ){
	if( editor->tree != NULL ){
		return gtk_xml_view_get_document( GTK_XML_VIEW(editor->tree));
	}
	return NULL;
}
xmlDoc*
gtk_xml_editor_set_document( GtkXmlEditor *editor, xmlDoc* doc ){
	GtkXmlEditorPrivate* priv = (GtkXmlEditorPrivate*)editor->private_data;
	xmlDoc* olddoc;
	if( doc == NULL ){
		doc = xmlNewDoc("1.0");
	}
	olddoc = gtk_xml_editor_get_document(editor);
	/*
	if( editor->tree != NULL ){
		gtk_container_remove( GTK_CONTAINER(priv->left), 
                                      GTK_WIDGET(editor->tree));
	}
	editor->tree = GTK_XML_TREE(gtk_xml_tree_new(doc));
	gtk_signal_connect(GTK_OBJECT(GTK_XML_VIEW(editor->tree)), 
                           "select_node", 
                           GTK_SIGNAL_FUNC(_xml_select_node_sig), editor );
	gtk_signal_connect(GTK_OBJECT(GTK_XML_VIEW(editor->tree)), 
                           "update_node", 
                           GTK_SIGNAL_FUNC(_xml_update_node_sig), editor );
	gtk_signal_connect(GTK_OBJECT(GTK_XML_VIEW(editor->tree)),
									"delete_node",
									GTK_SIGNAL_FUNC(_xml_editor_delete_node_sig), editor);
	gtk_container_add(GTK_CONTAINER(priv->left), GTK_WIDGET(editor->tree));
	gtk_xml_view_select_node(GTK_XML_VIEW(editor->tree),NULL);
	*/
	xml_editor_draw_node(editor,NULL);
	gtk_widget_show(GTK_WIDGET(editor->tree));
	return olddoc;
}
GtkXmlView*
gtk_xml_editor_get_view( GtkXmlEditor *editor ){
	return (GtkXmlView *)editor->tree;
}
void
gtk_xml_editor_set_view( GtkXmlEditor* editor, GtkXmlView* view ){
	GtkXmlEditorPrivate* priv;
	if( editor->tree != NULL ){
		gtk_container_remove( GTK_CONTAINER(priv->left), 
                                      GTK_WIDGET(editor->tree));
	}
	editor->tree = view;
	priv = (GtkXmlEditorPrivate*)editor->private_data;
	if( priv->left != NULL ){
		if( !GTK_IS_CONTAINER(priv->left) ){
			return;
		}
		gtk_signal_connect(GTK_OBJECT(GTK_XML_VIEW(editor->tree)), 
   	                        "select_node", 
   	                        GTK_SIGNAL_FUNC(_xml_select_node_sig), editor );
		gtk_signal_connect(GTK_OBJECT(GTK_XML_VIEW(editor->tree)), 
   	                        "update_node", 
   	                        GTK_SIGNAL_FUNC(_xml_update_node_sig), editor );
		gtk_signal_connect(GTK_OBJECT(GTK_XML_VIEW(editor->tree)),
										"delete_node",
										GTK_SIGNAL_FUNC(_xml_editor_delete_node_sig), editor);
		gtk_container_add(GTK_CONTAINER(priv->left), GTK_WIDGET(editor->tree));
		gtk_xml_view_select_node(GTK_XML_VIEW(editor->tree),NULL);
		gtk_widget_show(view);
	}
}

xmlNode*
gtk_xml_editor_get_current_node( GtkXmlEditor* editor ){
	return gtk_xml_view_get_selected_node( GTK_XML_VIEW(editor->tree));
}
