/**
 * $Id: modules.c,v 1.4 2001/05/16 21:37:57 makki Exp $
 * $Author: makki $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gnome-xml/tree.h"
#include "gnome-xml/parser.h"
#include "modules.h"


GModule*
mod_get_module( ModuleList* list, gchar* name ){
	if( list == NULL ){
		return NULL;
	}
	if( name == NULL ){
		return NULL;
	}
	return mod_get_module_info(list,name)->module;
}
ModuleInfo*
mod_get_module_info( ModuleList* list, gchar* name ){
	if( list == NULL ){
		return NULL;
	}
	if( name == NULL ){
		return NULL;
	}
	return (ModuleInfo*)g_hash_table_lookup( list->table, name );
}

ModuleList*
mod_new(){
	ModuleList* list = (ModuleList*)malloc(sizeof(ModuleList));
	list->table = g_hash_table_new(g_str_hash, g_str_equal);
	list->list = NULL;
	return list;
}
ModuleList*
mod_load_list( ModuleList* list, xmlDoc* doc ){
	xmlNode* el;
	if( doc == NULL ){
		doc = xmlNewDoc("1.0");
	}
	el = xmlDocGetRootElement(doc);
	if( el == NULL ){
		return list;
	}
	el = el->childs;
	while( el != NULL ){
		if( strcmp(el->name,"module") == 0 ){
			xmlNode* n = el->childs;
			ModuleInfo* info;
			info = mod_add_module(list, xmlGetProp(el, "lib") );
			if( info != NULL ){
				while( n != NULL ){
					if( strcmp(n->name, "attribute") == 0 ){
						char* name = xmlGetProp(n,"name");
						char* value = xmlNodeGetContent(n);
						mod_add_attribute( info, name, value );
					}
					n = n->next;
				}
			}else{
				show_error("Adding of module failed.");
				g_print("mod_add_module failed on module %s\n",xmlGetProp(el,"lib"));
			}
		}
		el = el->next;
	}
	return list;
}
ModuleInfo*
mod_add_attribute( ModuleInfo* info, gchar* name, gchar* value ){
	if( name == NULL ){
		return info;
	}
	if( value == NULL ){
		return info;
	}
	if( info == NULL ){
		return NULL;
	}
	if( info->attributes == NULL ){
		info->attributes = (gchar**)malloc( sizeof(gchar*)*3 );
		info->attributes[0] = g_strdup(name);
		info->attributes[1] = g_strdup(value);
		info->attributes[2] = NULL;
	}else{
		int size;
		for( size=0; info->attributes[size] != NULL; size++);
		info->attributes = (gchar**)realloc( info->attributes, (size + 2) * sizeof(gchar*) );
		info->attributes[size] = g_strdup(name);
		info->attributes[size+1] = g_strdup(value);
		info->attributes[size+2] = NULL;
	}
	return info;
}
gchar*
mod_get_attribute( ModuleInfo* info, gchar* name ){
	if( info == NULL ){
		return NULL;
	}
	if( name == NULL ){
		return NULL;
	}
	if( info->attributes != NULL ){
		int i=0;
		while( info->attributes[i] != NULL ){
			if( strcmp( info->attributes[i], name ) == 0 ){
				return info->attributes[i+1];
			}
			i+=2;
		}
	}
	return NULL;
}
/** Don't forget to free this structure
 */
gchar**
mod_get_attributes( ModuleInfo* info, gchar* name ){
	int count = 0;
	gchar** list = (gchar**)malloc(sizeof(gchar*));
	list[0] = NULL;
	if( info == NULL ){
		return list;
	}
	if( name == NULL ){
		return list;
	}
	if( info->attributes != NULL ){
		int i=0;
		while( info->attributes[i] != NULL ){
			if( strcmp( info->attributes[i], name ) == 0 ){
				list[count] = info->attributes[i+1];
				count++;
				list = (gchar**)realloc( list, (count+1) * sizeof(gchar*) );
				list[count] = NULL;
			}else{
		//		g_print( "is not equal: %s,%s\n",info->attributes[i], name );
			}
			i+=2;
		}
	}
	return list;
}
ModuleInfo*
mod_add_module( ModuleList* list, gchar* filename ){
	ModuleInfo* info = (ModuleInfo*)malloc(sizeof(ModuleInfo));
	if( list == NULL ){
		return NULL;
	}
	if( filename == NULL ){
		free(info);
		return NULL;
	}
	info->module = g_module_open(filename, G_MODULE_BIND_MASK);
	if( info->module == NULL ){
		show_error("Failed to open module.");
		g_print("Failed to open module %s\n",filename);
		free(info);
		return NULL;
	}
	if( list == NULL ){
		free(info);
		return NULL;
	}
	info->name = g_strdup(filename);
	info->container = list;
	/** init */
	info->enabled = FALSE;
	info->help = NULL;
	info->attributes = NULL;
	info->user_data = NULL;
	/** module_init */
	if( g_module_symbol( info->module, "module_init", &(info->init) ) ){
		info->init(&info);
		g_hash_table_insert( list->table, filename, info );
		list->list = g_list_append( list->list, info );
	}else{
		g_module_close(info->module);
		free(info);
		return NULL;
	}
	return info;
}
